@@ -2,8 +2,12 @@
  * xslt.c: Implemetation of an XSL Transformation 1.0 engine
  *
  * Reference:
+ *   XSLT specification
  *   http://www.w3.org/TR/1999/REC-xslt-19991116
  *
+ *   Associating Style Sheets with XML documents
+ *   http://www.w3.org/1999/06/REC-xml-stylesheet-19990629
+ *
  * See Copyright for the status of this software.
  *
  * daniel@veillard.com
@@ -273,6 +277,7 @@ xsltFreeTemplate(xsltTemplatePtr template) {
     if (template->nameURI) xmlFree(template->nameURI);
     if (template->mode) xmlFree(template->mode);
     if (template->modeURI) xmlFree(template->modeURI);
+    if (template->inheritedNs) xmlFree(template->inheritedNs);
     memset(template, -1, sizeof(xsltTemplate));
     xmlFree(template);
 }
@@ -403,10 +408,95 @@ xsltFreeStylesheet(xsltStylesheetPtr sheet)
  *									*
  ************************************************************************/
 
+/**
+ * xsltGetInheritedNsList:
+ * @style:  the stylesheet
+ * @template: the template
+ * @node:  the current node
+ *
+ * Search all the namespace applying to a given element except the ones 
+ * from excluded output prefixes currently in scope. Initialize the
+ * template inheritedNs list with it.
+ *
+ * Returns the number of entries found
+ */
+static int
+xsltGetInheritedNsList(xsltStylesheetPtr style,
+	               xsltTemplatePtr template,
+	               xmlNodePtr node)
+{
+    xmlNsPtr cur;
+    xmlNsPtr *ret = NULL;
+    int nbns = 0;
+    int maxns = 10;
+    int i;
+
+    if ((style == NULL) || (template == NULL) || (node == NULL) ||
+	(template->inheritedNsNr != 0) || (template->inheritedNs != NULL))
+	return(0);
+    while (node != NULL) {
+        if (node->type == XML_ELEMENT_NODE) {
+            cur = node->nsDef;
+            while (cur != NULL) {
+		if (xmlStrEqual(cur->href, XSLT_NAMESPACE))
+		    goto skip_ns;
+		for (i = 0;i < style->exclPrefixNr;i++) {
+		    if (xmlStrEqual(cur->href, style->exclPrefixTab[i]))
+			goto skip_ns;
+		}
+                if (ret == NULL) {
+                    ret =
+                        (xmlNsPtr *) xmlMalloc((maxns + 1) *
+                                               sizeof(xmlNsPtr));
+                    if (ret == NULL) {
+                        xmlGenericError(xmlGenericErrorContext,
+                                        "xmlGetNsList : out of memory!\n");
+                        return(0);
+                    }
+                    ret[nbns] = NULL;
+                }
+                for (i = 0; i < nbns; i++) {
+                    if ((cur->prefix == ret[i]->prefix) ||
+                        (xmlStrEqual(cur->prefix, ret[i]->prefix)))
+                        break;
+                }
+                if (i >= nbns) {
+                    if (nbns >= maxns) {
+                        maxns *= 2;
+                        ret = (xmlNsPtr *) xmlRealloc(ret,
+                                                      (maxns +
+                                                       1) *
+                                                      sizeof(xmlNsPtr));
+                        if (ret == NULL) {
+                            xmlGenericError(xmlGenericErrorContext,
+                                            "xmlGetNsList : realloc failed!\n");
+                            return(0);
+                        }
+                    }
+                    ret[nbns++] = cur;
+                    ret[nbns] = NULL;
+                }
+skip_ns:
+                cur = cur->next;
+            }
+        }
+        node = node->parent;
+    }
+    if (nbns != 0) {
+#ifdef WITH_XSLT_DEBUG_PARSING
+        xsltGenericDebug(xsltGenericDebugContext,
+                         "template has %d inherited namespaces\n", nbns);
+#endif
+	template->inheritedNsNr = nbns;
+	template->inheritedNs = ret;
+    }
+    return (nbns);
+}
+
 /**
  * xsltParseStylesheetOutput:
  * @style:  the XSLT stylesheet
- * @template:  the "output" element
+ * @cur:  the "output" element
  *
  * parse an XSLT stylesheet output element and record
  * information related to the stylesheet output
@@ -684,7 +774,7 @@ xsltParseStylesheetDecimalFormat(xsltStylesheetPtr style, xmlNodePtr cur)
 /**
  * xsltParseStylesheetPreserveSpace:
  * @style:  the XSLT stylesheet
- * @template:  the "preserve-space" element
+ * @cur:  the "preserve-space" element
  *
  * parse an XSLT stylesheet preserve-space element and record
  * elements needing preserving
@@ -797,7 +887,7 @@ xsltParseStylesheetExtPrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 /**
  * xsltParseStylesheetStripSpace:
  * @style:  the XSLT stylesheet
- * @template:  the "strip-space" element
+ * @cur:  the "strip-space" element
  *
  * parse an XSLT stylesheet strip-space element and record
  * elements needing stripping
@@ -893,13 +983,13 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 		ns = xmlSearchNs(style->doc, cur, prefix);
 	    if (ns == NULL) {
 		xsltGenericError(xsltGenericErrorContext,
-	    "xsl:extension-element-prefix : undefined namespace %s\n",
+	    "xsl:exclude-result-prefixes : undefined namespace %s\n",
 	                         prefix);
 		style->warnings++;
 	    } else {
 #ifdef WITH_XSLT_DEBUG_PARSING
 		xsltGenericDebug(xsltGenericDebugContext,
-		    "add extension prefix %s\n", prefix);
+		    "exclude result prefix %s\n", prefix);
 #endif
 		exclPrefixPush(style, (xmlChar *) ns->href);
 		nb++;
@@ -958,18 +1048,42 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    /*
 	     * Remove excluded prefixes
 	     */
-	    if ((cur->ns != NULL) && (style->exclPrefixNr > 0)) {
-		int i;
+	    if ((cur->nsDef != NULL) && (style->exclPrefixNr > 0)) {
+		xmlNsPtr ns = cur->nsDef, prev = NULL, next;
+		xmlNodePtr root = NULL;
+		int i, moved;
 
-		for (i = 0;i < style->exclPrefixNr;i++) {
-		    if (xmlStrEqual(cur->ns->href, style->exclPrefixTab[i])) {
-			for (;exclPrefixes > 0;exclPrefixes--)
-			    prefix = exclPrefixPop(style);
-			delete = cur;
-			goto skip_children;
+		root = xmlDocGetRootElement(cur->doc);
+		if ((root != NULL) && (root != cur)) {
+		    while (ns != NULL) {
+			moved = 0;
+			next = ns->next;
+			for (i = 0;i < style->exclPrefixNr;i++) {
+			    if (xmlStrEqual(ns->href,
+					    style->exclPrefixTab[i])) {
+				/*
+				 * Move the namespace definition on the root
+				 * element to avoid duplicating it without
+				 * loosing it.
+				 */
+				if (prev == NULL) {
+				    cur->nsDef = ns->next;
+				} else {
+				    prev->next = ns->next;
+				}
+				ns->next = root->nsDef;
+				root->nsDef = ns;
+				moved = 1;
+				break;
+			    }
+			}
+			if (moved == 0)
+			    prev = ns;
+			ns = next;
 		    }
 		}
 	    }
+
 	    /*
 	     * If we have prefixes locally, recurse and pop them up when
 	     * going back
@@ -1038,7 +1152,7 @@ skip_children:
  * xsltGatherNamespaces:
  * @style:  the XSLT stylesheet
  *
- * Browse the stylesheet and buit the namspace hash table which
+ * Browse the stylesheet and build the namspace hash table which
  * will be used for XPath interpretation. If needed do a bit of normalization
  */
 
@@ -1169,7 +1283,7 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xsltTemplatePtr ret,
 			} else if (!xmlStrEqual(prop,
 						(const xmlChar *)"no")){
 			    xsltGenericError(xsltGenericErrorContext,
-	     "xslt:text: disable-output-escaping allow only yes or no\n");
+	     "xsl:text: disable-output-escaping allows only yes or no\n");
 			    style->warnings++;
 
 			}
@@ -1177,9 +1291,8 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xsltTemplatePtr ret,
 		    }
 
 		    while (text != NULL) {
-			if (((text->type != XML_TEXT_NODE) &&
-			     (text->type != XML_CDATA_SECTION_NODE)) ||
-			    (text->next != NULL)) {
+			if ((text->type != XML_TEXT_NODE) &&
+			     (text->type != XML_CDATA_SECTION_NODE)) {
 			    xsltGenericError(xsltGenericErrorContext,
 		 "xsltParseTemplateContent: xslt:text content problem\n");
 			    style->errors++;
@@ -1247,7 +1360,7 @@ skip_children:
     if (delete != NULL) {
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
-	 "xsltParseStylesheetTemplate: removing text\n");
+	 "xsltParseTemplateContent: removing text\n");
 #endif
 	xmlUnlinkNode(delete);
 	xmlFreeNode(delete);
@@ -1265,7 +1378,7 @@ skip_children:
     }
 
     /*
-     * Browse the remaining of the template
+     * Browse the remainder of the template
      */
     while (cur != NULL) {
 	if ((IS_XSLT_ELEM(cur)) && (IS_XSLT_NAME(cur, "param"))) {
@@ -1273,7 +1386,7 @@ skip_children:
 
             cur = cur->next;
 	    xsltGenericError(xsltGenericErrorContext,
-		"xsltParseStylesheetTop: ignoring misplaced param element\n");
+		"xsltParseTemplateContent: ignoring misplaced param element\n");
 	    style->warnings++;
 	    xmlUnlinkNode(param);
 	    xmlFreeNode(param);
@@ -1323,7 +1436,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 	}
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
-	     "xslt:key: name %s\n", name);
+	     "xsltParseStylesheetKey: name %s\n", name);
 #endif
     } else {
 	xsltGenericError(xsltGenericErrorContext,
@@ -1379,6 +1492,7 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xmlChar *mode = NULL;
     xmlChar *modeURI = NULL;
     double  priority;
+    int exclPrefixes;
 
     if (template == NULL)
 	return;
@@ -1393,6 +1507,16 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     style->templates = ret;
     ret->style = style;
 
+    /*
+     * Check excluded prefixes
+     */
+    exclPrefixes = xsltParseStylesheetExcludePrefix(style, template);
+
+    /*
+     * Get inherited namespaces
+     */
+    xsltGetInheritedNsList(style, ret, template);
+
     /*
      * Get arguments
      */
@@ -1411,7 +1535,7 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 	}
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
-	     "xslt:template: mode %s\n", mode);
+	     "xsltParseStylesheetTemplate: mode %s\n", mode);
 #endif
     } else {
 	mode = NULL;
@@ -1459,6 +1583,8 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xsltAddTemplate(style, ret, mode, modeURI);
 
 error:
+    for (;exclPrefixes > 0;exclPrefixes--)
+	exclPrefixPop(style);
     if (mode != NULL)
 	xmlFree(mode);
     if (modeURI != NULL)
@@ -1738,3 +1864,218 @@ xsltParseStylesheetFile(const xmlChar* filename) {
     return(ret);
 }
 
+/************************************************************************
+ *									*
+ *			Handling of Stylesheet PI			*
+ *									*
+ ************************************************************************/
+
+#define CUR (*cur)
+#define SKIP(val) cur += (val)
+#define NXT(val) cur[(val)]
+#define SKIP_BLANKS						\
+    while (IS_BLANK(CUR)) NEXT
+#define NEXT ((*cur) ?  cur++ : cur)
+
+/**
+ * xsltParseStylesheetPI:
+ * value: the value of the PI
+ *
+ * This function checks that the type is text/xml and extracts
+ * the URI-Reference for the stylesheet
+ *
+ * Returns the URI-Reference for the stylesheet or NULL (it need to
+ *         be freed by the caller)
+ */
+static xmlChar *
+xsltParseStylesheetPI(const xmlChar *value) {
+    const xmlChar *cur;
+    const xmlChar *start;
+    xmlChar *val;
+    xmlChar tmp;
+    xmlChar *href = NULL;
+    int isXml = 0;
+
+    if (value == NULL)
+	return(NULL);
+
+    cur = value;
+    while (CUR != 0) {
+	SKIP_BLANKS;
+	if ((CUR == 't') && (NXT(1) == 'y') && (NXT(2) == 'p') &&
+	    (NXT(3) == 'e')) {
+	    SKIP(4);
+	    SKIP_BLANKS;
+	    if (CUR != '=')
+		continue;
+	    NEXT;
+	    if ((CUR != '\'') && (CUR != '"'))
+		continue;
+	    tmp = CUR;
+	    NEXT;
+	    start = cur;
+	    while ((CUR != 0) && (CUR != tmp))
+		NEXT;
+	    if (CUR != tmp)
+		continue;
+	    val = xmlStrndup(start, cur - start);
+	    NEXT;
+	    if (val == NULL) 
+		return(NULL);
+	    if ((xmlStrcasecmp(val, BAD_CAST "text/xml")) &&
+		(xmlStrcasecmp(val, BAD_CAST "text/xsl"))) {
+                xmlFree(val);
+		break;
+	    }
+	    isXml = 1;
+	    xmlFree(val);
+	} else if ((CUR == 'h') && (NXT(1) == 'r') && (NXT(2) == 'e') &&
+	    (NXT(3) == 'f')) {
+	    SKIP(4);
+	    SKIP_BLANKS;
+	    if (CUR != '=')
+		continue;
+	    NEXT;
+	    if ((CUR != '\'') && (CUR != '"'))
+		continue;
+	    tmp = CUR;
+	    NEXT;
+	    start = cur;
+	    while ((CUR != 0) && (CUR != tmp))
+		NEXT;
+	    if (CUR != tmp)
+		continue;
+	    if (href == NULL)
+		href = xmlStrndup(start, cur - start);
+	    NEXT;
+	} else {
+	    while ((CUR != 0) && (!IS_BLANK(CUR)))
+		NEXT;
+	}
+            
+    }
+
+    if (!isXml) {
+	if (href != NULL)
+	    xmlFree(href);
+	href = NULL;
+    }
+    return(href);
+}
+
+/**
+ * xsltLoadStylesheetPI:
+ * doc:  a document to process
+ *
+ * This function tries to locate the stylesheet PI in the given document
+ * If found, and if contained within the document, it will extract 
+ * that subtree to build the stylesheet to process @doc (doc itself will
+ * be modified). If found but referencing an external document it will
+ * attempt to load it and generate a stylesheet from it. In both cases,
+ * the resulting stylesheet and the document need to be freed once the
+ * transformation is done.
+ *
+ * Returns a new XSLT stylesheet structure or NULL if not found.
+ */
+xsltStylesheetPtr
+xsltLoadStylesheetPI(xmlDocPtr doc) {
+    xmlNodePtr child;
+    xsltStylesheetPtr ret = NULL;
+    xmlChar *href = NULL;
+    xmlURIPtr URI;
+
+    if (doc == NULL)
+	return(NULL);
+
+    /*
+     * Find the text/xml stylesheet PI id any before the root
+     */
+    child = doc->children;
+    while ((child != NULL) && (child->type != XML_ELEMENT_NODE)) {
+	if ((child->type == XML_PI_NODE) &&
+	    (xmlStrEqual(child->name, BAD_CAST "xml-stylesheet"))) {
+	    href = xsltParseStylesheetPI(child->content);
+	    if (href != NULL)
+		break;
+	}
+	child = child->next;
+    }
+
+    /*
+     * If found check the href to select processing
+     */
+    if (href != NULL) {
+#ifdef WITH_XSLT_DEBUG_PARSING
+	xsltGenericDebug(xsltGenericDebugContext,
+		"xsltLoadStylesheetPI : found PI href=%s\n", href);
+#endif
+	URI = xmlParseURI((const char *) href);
+	if (URI == NULL) {
+	    xsltGenericError(xsltGenericErrorContext,
+		    "xml-stylesheet : href %s is not valid\n", href);
+	    xmlFree(href);
+	    return(NULL);
+	}
+	if ((URI->fragment != NULL) && (URI->scheme == NULL) &&
+            (URI->opaque == NULL) && (URI->authority == NULL) &&
+            (URI->server == NULL) && (URI->user == NULL) &&
+            (URI->path == NULL) && (URI->query == NULL)) {
+	    xmlAttrPtr ID;
+
+#ifdef WITH_XSLT_DEBUG_PARSING
+	    xsltGenericDebug(xsltGenericDebugContext,
+		    "xsltLoadStylesheetPI : Reference to ID %s\n", href);
+#endif
+	    if (URI->fragment[0] == '#')
+		ID = xmlGetID(doc, (const xmlChar *) &(URI->fragment[1]));
+	    else
+		ID = xmlGetID(doc, (const xmlChar *) URI->fragment);
+	    if (ID == NULL) {
+		xsltGenericError(xsltGenericErrorContext,
+		    "xml-stylesheet : no ID %s found\n", URI->fragment);
+	    } else {
+		xmlDocPtr fake;
+		xmlNodePtr subtree;
+
+		/*
+		 * move the subtree in a new document passed to
+		 * the stylesheet analyzer
+		 */
+		subtree = ID->parent;
+		fake = xmlNewDoc(NULL);
+		if (fake != NULL) {
+		    xmlUnlinkNode(subtree);
+		    xmlAddChild((xmlNodePtr) fake, subtree);
+		    ret = xsltParseStylesheetDoc(fake);
+		    if (ret == NULL)
+			xmlFreeDoc(fake);
+		}
+	    }
+	} else {
+	    xmlChar *URL;
+
+	    /*
+	     * Reference to an external stylesheet
+	     */
+	    URL = xmlBuildURI(href, xmlNodeGetBase(doc, (xmlNodePtr) doc));
+	    if (URL != NULL) {
+#ifdef WITH_XSLT_DEBUG_PARSING
+		xsltGenericDebug(xsltGenericDebugContext,
+			"xsltLoadStylesheetPI : fetching %s\n", URL);
+#endif
+		ret = xsltParseStylesheetFile(URL);
+		xmlFree(URL);
+	    } else {
+#ifdef WITH_XSLT_DEBUG_PARSING
+		xsltGenericDebug(xsltGenericDebugContext,
+			"xsltLoadStylesheetPI : fetching %s\n", href);
+#endif
+		ret = xsltParseStylesheetFile(href);
+	    }
+	}
+	xmlFreeURI(URI);
+	xmlFree(href);
+    }
+    return(ret);
+}
+
