@@ -54,6 +54,13 @@ struct _xsltTemplate {
     xmlChar *modeURI;	/* the URI part of the mode QName */
     xmlNodePtr content;	/* the template replacement value */
     xmlNodePtr elem;	/* the source element */
+
+    int inheritedNsNr;  /* number of inherited namespaces */
+    xmlNsPtr *inheritedNs;/* inherited non-excluded namespaces */
+
+    /* Profiling informations */
+    int nbCalls;        /* the number of time the template was called */
+    unsigned long time; /* the time spent in this template */
 };
 
 /**
@@ -120,7 +127,8 @@ typedef xsltStylePreComp *xsltStylePreCompPtr;
  * stylesheet language like xsl:if or xsl:apply-templates.
  */
 typedef void (*xsltTransformFunction) (xsltTransformContextPtr ctxt,
-	                               xmlNodePtr node, xmlNodePtr inst,
+	                               xmlNodePtr node,
+				       xmlNodePtr inst,
 			               xsltStylePreCompPtr comp);
 
 typedef enum {
@@ -309,7 +317,7 @@ struct _xsltStylesheet {
     int warnings;		/* number of warnings found at compilation */
     int errors;			/* number of errors found at compilation */
 
-    xmlChar  *exclPrefix;	/* array of excluded prefixes */
+    xmlChar  *exclPrefix;	/* last excluded prefixes */
     xmlChar **exclPrefixTab;	/* array of excluded prefixes */
     int       exclPrefixNr;	/* number of excluded prefixes in scope */
     int       exclPrefixMax;	/* size of the array */
@@ -343,6 +351,7 @@ struct _xsltTransformContext {
     int               varsNr;		/* Nb of variable list in the stack */
     int               varsMax;		/* Size of the variable list stack */
     xsltStackElemPtr *varsTab;		/* the variable list stack */
+    int               varsBase;		/* the var base for current templ */
 
     /*
      * Extensions
@@ -377,6 +386,12 @@ struct _xsltTransformContext {
     int xinclude;			/* should XInclude be processed */
 
     const char *      outputFile;	/* the output URI if known */
+
+    int profile;                        /* is this run profiled */
+    long             prof;		/* the current profiled value */
+    int              profNr;		/* Nb of templates in the stack */
+    int              profMax;		/* Size of the templtaes stack */
+    long            *profTab;		/* the profile template stack */
 };
 
 /**
@@ -421,6 +436,7 @@ xsltStylesheetPtr	xsltParseStylesheetProcess(xsltStylesheetPtr ret,
 void			xsltParseStylesheetOutput(xsltStylesheetPtr style,
 						  xmlNodePtr cur);
 xsltStylesheetPtr	xsltParseStylesheetDoc	(xmlDocPtr doc);
+xsltStylesheetPtr	xsltLoadStylesheetPI	(xmlDocPtr doc);
 void 			xsltNumberFormat	(xsltTransformContextPtr ctxt,
 						 xsltNumberDataPtr data,
 						 xmlNodePtr node);
